package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.common.utils.UploadImage;
import com.huimv.farm.damsubsidy.entity.BaseAnimal;
import com.huimv.farm.damsubsidy.entity.BillIsolated;
import com.huimv.farm.damsubsidy.entity.BillLandingInspection;
import com.huimv.farm.damsubsidy.mapper.BaseAnimalMapper;
import com.huimv.farm.damsubsidy.mapper.BillIsolatedMapper;
import com.huimv.farm.damsubsidy.mapper.BillLandingInspectionMapper;
import com.huimv.farm.damsubsidy.service.IBillIsolatedService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 落地检 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillIsolatedServiceImpl extends ServiceImpl<BillIsolatedMapper, BillIsolated> implements IBillIsolatedService {

    @Resource
    private BillLandingInspectionMapper landingInspectionMapper;

    @Resource
    private BaseAnimalMapper baseAnimalMapper;

    @Override
    @Transactional
    public Result addIsolation(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String inspectionNum = paramsMap.get("inspectionNum");
        String strIsolated = paramsMap.get("strIsolated");
        String dayIsolated = paramsMap.get("dayIsolated");
        String endIsolated = paramsMap.get("endIsolated");

        BillIsolated billIsolated = new BillIsolated();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        billIsolated.setInspectionName(TokenSign.getUserName(httpServletRequest)+"的隔离信息");
        billIsolated.setInspectionNum(inspectionNum);
        billIsolated.setStrIsolated(LocalDateTime.parse(strIsolated,formatter));
        billIsolated.setDayIsolated(Integer.parseInt(dayIsolated));
        billIsolated.setEndIsolated(LocalDateTime.parse(endIsolated,formatter));
        billIsolated.setCreateTime(DateTime.now());
        billIsolated.setCreateUser(TokenSign.getUserName(httpServletRequest));

        BillLandingInspection billLandingInspection = landingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", inspectionNum).ne("inspection_sch", 5));
        String eartags = billLandingInspection.getEartags();

        if (eartags.contains(";")){
            String[] eartag = eartags.split(";");
            for (String input : eartag) {
                if (input.contains("-")){
                    String start = input.substring(0, input.indexOf("-"));
                    String end = input.substring(input.indexOf("-") + 1);
                    String substring = start.substring(0, start.length() - end.length());
                    String finish = substring+end;
                    List<Integer> numbers = new ArrayList<>();
                    for (int i = Integer.parseInt(start); i <= Integer.parseInt(finish); i++) {
                        numbers.add(i);
                    }
                    for (Integer number : numbers) {
                        String earatgNo = number.toString();
                        BaseAnimal baseAnimal = new BaseAnimal();
                        baseAnimal.setEaratgNo(earatgNo);
                        baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
                        baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
                        baseAnimal.setBelongToUser(TokenSign.getUserName(httpServletRequest));
                        baseAnimal.setCreateUser(TokenSign.getUserName(httpServletRequest));
                        baseAnimal.setCreateTime(new Date());
                        baseAnimalMapper.insert(baseAnimal);
                    }
                }else {
                    BaseAnimal baseAnimal = new BaseAnimal();
                    baseAnimal.setEaratgNo(input);
                    baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
                    baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
                    baseAnimal.setBelongToUser(TokenSign.getUserName(httpServletRequest));
                    baseAnimal.setCreateUser(TokenSign.getUserName(httpServletRequest));
                    baseAnimal.setCreateTime(new Date());
                    baseAnimalMapper.insert(baseAnimal);
                }
            }
        }else {
            if (eartags.contains("-")){
                String start = eartags.substring(0, eartags.indexOf("-"));
                String end = eartags.substring(eartags.indexOf("-") + 1);
                String substring = start.substring(0, start.length() - end.length());
                String finish = substring+end;
                List<Integer> numbers = new ArrayList<>();
                for (int i = Integer.parseInt(start); i <= Integer.parseInt(finish); i++) {
                    numbers.add(i);
                }
                for (Integer number : numbers) {
                    String earatgNo = number.toString();
                    BaseAnimal baseAnimal = new BaseAnimal();
                    baseAnimal.setEaratgNo(earatgNo);
                    baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
                    baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
                    baseAnimal.setBelongToUser(TokenSign.getUserName(httpServletRequest));
                    baseAnimal.setCreateUser(TokenSign.getUserName(httpServletRequest));
                    baseAnimal.setCreateTime(new Date());
                    baseAnimalMapper.insert(baseAnimal);
                }
            }else {
                BaseAnimal baseAnimal = new BaseAnimal();
                baseAnimal.setEaratgNo(eartags);
                baseAnimal.setAnimalType(billLandingInspection.getAnimalType());
                baseAnimal.setInspectionNum(billLandingInspection.getInspectionNum());
                baseAnimal.setBelongToUser(TokenSign.getUserName(httpServletRequest));
                baseAnimal.setCreateUser(TokenSign.getUserName(httpServletRequest));
                baseAnimal.setCreateTime(new Date());
                baseAnimalMapper.insert(baseAnimal);
            }
        }

        billLandingInspection.setInspectionSch(2);
        billLandingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        billLandingInspection.setReviewedTime(DateTime.now());
        landingInspectionMapper.updateById(billLandingInspection);
        this.save(billIsolated);

        return Result.SUCCESS();


    }

    @Override
    public Result listIsolation(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        if (type.equals("1")){
            return new Result(ResultCode.SUCCESS,this.list(new QueryWrapper<BillIsolated>().ge("str_isolated",LocalDate.now()).le("end_isolated",LocalDate.now())));
        }else {
            return new Result(ResultCode.SUCCESS,this.list(new QueryWrapper<BillIsolated>().lt("end_isolated",LocalDate.now())));
        }
    }

    @Override
    public Result delayedIsolation(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String day = paramsMap.get("day");
        BillIsolated billIsolated = this.getById(id);
        billIsolated.setDayIsolated(billIsolated.getDayIsolated()+Integer.parseInt(day));
        billIsolated.setEndIsolated(billIsolated.getEndIsolated().plusDays(Integer.parseInt(day)));
        this.updateById(billIsolated);
        return Result.SUCCESS();
    }

    @Override
    @Transactional
    public Result cancelIsolation(HttpServletRequest httpServletRequest, MultipartFile image, String id, String isDead, String deaths) {
        String imgname = "成功";
        String originalFilename = image.getOriginalFilename();
        String filenameExtension = StringUtils.getFilenameExtension(originalFilename);
        String path = DateUtil.format(new Date(),"yyyy-MM");
        try {
            InputStream inputStream = image.getInputStream();
            BufferedInputStream in = new BufferedInputStream(inputStream);
            ByteArrayOutputStream out = new ByteArrayOutputStream(1024);

            byte[] temp = new byte[1024];
            int size = 0;
            while ((size = in.read(temp)) != -1) {
                out.write(temp, 0, size);
            }
            in.close();
            byte[] content = out.toByteArray();
            imgname = UUID.randomUUID()+"."+filenameExtension;
            UploadImage.sshSftp(content,path, imgname);
        } catch (Exception e) {
            e.printStackTrace();
        }
        String content = "http://139.9.167.178/images"+path +"/"+imgname;
        BillIsolated isolation = this.getById(id);
        isolation.setIsolatedPicUrl(content);
        isolation.setIsolatedSch(2);
        if ("1".equals(isDead)){
            BillIsolated billIsolated = this.getById(id);
            List<BaseAnimal> baseAnimals = baseAnimalMapper.selectList(new QueryWrapper<BaseAnimal>().eq("inspection_num", billIsolated.getInspectionNum()));
            for (BaseAnimal baseAnimal : baseAnimals) {
                baseAnimal.setAnimalStatus(1);
                baseAnimalMapper.updateById(baseAnimal);
            }
            isolation.setIsDead(1);
            isolation.setDeaths(deaths);
            String[] eartag = deaths.split(";");
            for (String s : eartag) {
                if (!s.contains("-")){
                    BaseAnimal baseAnimal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", s));
                    baseAnimal.setAnimalStatus(4);
                    baseAnimalMapper.updateById(baseAnimal);
                }else {
                    String start = s.substring(0, s.indexOf("-"));
                    String end = s.substring(s.indexOf("-") + 1);
                    String substring = start.substring(0, start.length() - end.length());
                    String begin = start.substring(end.length()+1);
                    List<Integer> numbers = new ArrayList<>();
                    for (int i = Integer.parseInt(begin); i <= Integer.parseInt(end); i++) {
                        numbers.add(i);
                    }
                    for (Integer number : numbers) {
                        String earatgNo = substring + number.toString();
                        BaseAnimal animal = baseAnimalMapper.selectOne(new QueryWrapper<BaseAnimal>().eq("earatg_no", earatgNo));
                        animal.setIsLoated(4);
                        baseAnimalMapper.updateById(animal);
                    }
                }
            }
        }else {
            isolation.setIsDead(0);
        }
        BillLandingInspection billLandingInspection = landingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", isolation.getInspectionNum()));
        billLandingInspection.setInspectionSch(3);
        landingInspectionMapper.updateById(billLandingInspection);
        this.save(isolation);
        return Result.SUCCESS();
    }

}
